﻿Imports MindFusion.Diagramming.Wpf.Layout
Imports System.Windows.Media.Effects

'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'


''' <summary>
''' Interaction logic for MainWindow.xaml
''' </summary>
Partial Public Class MainWindow
    Inherits Window
    Public Sub New()
        InitializeComponent()

        diagram.AllowDrop = True
        diagram.DocumentPlane.AllowDrop = True

        diagram.NodeEffects.Add(New GlassEffect())
    End Sub

    Private treeLayout As MindFusion.Diagramming.Wpf.Layout.TreeLayout

    Private Sub Rearrange()
        If treeLayout Is Nothing Then
            treeLayout = New MindFusion.Diagramming.Wpf.Layout.TreeLayout(rootNode, TreeLayoutType.Centered, False, TreeLayoutLinkType.Rounded, TreeLayoutDirections.TopToBottom, 40, _
             20, True, New Size(10, 10))
        End If

        treeLayout.Arrange(diagram)
    End Sub

    Private Shadows Function AddChild(ByVal node As DiagramNode) As ShapeNode
        ' create the new node and add it to the parent group
        ' so its position is updated when the parent is moved
        Dim childNode = New ShapeNode(diagram) With { _
         .Bounds = New Rect(New Point(), NodeSize), _
         .AllowDrop = True _
        }
        childNode.Effect = New DropShadowEffect() With {.Opacity = 0.25}
        diagram.Nodes.Add(childNode)
        childNode.AttachTo(node, AttachToNode.TopLeft)

        ' link the parent node with the child
        Dim link = New DiagramLink(diagram, node, childNode)
        diagram.Links.Add(link)

        ' update the counter
        uniqueId += 1
        childNode.Tag = uniqueId

        ' rearrange the tree
        Rearrange()

        ' select the new node
        diagram.Selection.Change(childNode)

        Return childNode
    End Function

    Private rootNode As ShapeNode
    Private currNode As ShapeNode

    Private Sub OnImage1MouseDown(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
        DragDrop.DoDragDrop(image1, image1, DragDropEffects.Copy)
    End Sub

    Private Sub OnImage2MouseDown(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
        DragDrop.DoDragDrop(image2, image2, DragDropEffects.Copy)
    End Sub

    Private Sub diagram_DragOver(ByVal sender As Object, ByVal e As DragEventArgs)
        ' we have the screen point of the mouse, but need it in document C.S.
        Dim p As Point = e.GetPosition(diagram)
        Dim pt As Point = diagram.ClientToDoc(p)

        ' allow drag and drop if there is a box
        Dim box As ShapeNode = TryCast(diagram.GetNodeAt(pt), ShapeNode)
        If box IsNot Nothing Then
            e.Effects = DragDropEffects.Copy
        Else
            e.Effects = DragDropEffects.None
        End If
    End Sub

    Private Sub diagram_Drop(ByVal sender As Object, ByVal e As DragEventArgs)
        e.Handled = True
        Dim pt As Point = e.GetPosition(diagram.DocumentPlane)

        ' allow drag and drop if there is a box
        Dim node = TryCast(diagram.GetNodeAt(pt), ShapeNode)
        If node IsNot Nothing Then
            ' add a child to the tree
            Dim newNode As ShapeNode = AddChild(node)
            newNode.TextAlignment = TextAlignment.Right

            ' show the node tag
            newNode.Text = "id: " & newNode.Tag

            ' if an image has been dropped
            If e.Data.GetDataPresent(GetType(Image)) Then
                ' set as a node icon
                newNode.ImageAlign = ImageAlign.MiddleLeft
                newNode.Image = DirectCast(e.Data.GetData(GetType(Image)), Image).Source
            End If
        End If
    End Sub

    Private uniqueId As Integer

    Private Sub diagram_NodeClicked(ByVal sender As Object, ByVal e As NodeEventArgs)
        ' display context menu on right-click
        If e.MouseButton = MindFusion.Diagramming.Wpf.MouseButton.Right Then
            currNode = TryCast(e.Node, ShapeNode)

            ' display context menu for the node
            Dim cmNode As ContextMenu = CreateNodeMenu()
            cmNode.Placement = System.Windows.Controls.Primitives.PlacementMode.Mouse
            cmNode.IsOpen = True
        End If
    End Sub

    Private Sub diagram_NodeDeleting(ByVal sender As Object, ByVal e As NodeValidationEventArgs)
        If e.Node Is rootNode Then
            e.Cancel = True
        End If
    End Sub

    Private Sub diagram_NodeCreated(ByVal sender As Object, ByVal e As NodeEventArgs)
        Dim rc As Rect = e.Node.Bounds
        diagram.Nodes.Remove(e.Node)

        Dim child As ShapeNode = AddChild(rootNode)
        child.Resize(rc.Width, rc.Height)
        child.Text = "right-click me!"
    End Sub

    Private Function CreateNodeMenu() As ContextMenu
        Dim cmNode As New ContextMenu()

        Dim miProperties As New MenuItem()
        miProperties.Header = "Add child"
        AddHandler miProperties.Click, AddressOf miProperties_Click
        cmNode.Items.Add(miProperties)

        Return cmNode
    End Function

    Private Sub miProperties_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
        If currNode IsNot Nothing Then
            AddChild(currNode)
        End If
        currNode = Nothing
    End Sub

    Private Sub diagram_LinkModifying(ByVal sender As Object, ByVal e As LinkValidationEventArgs)
        e.Cancel = True
    End Sub

    Private Sub diagram_LinkCreating(ByVal sender As Object, ByVal e As LinkValidationEventArgs)
        e.Cancel = True
    End Sub

    Private Sub diagram_LinkSelecting(ByVal sender As Object, ByVal e As LinkValidationEventArgs)
        e.Cancel = True
    End Sub

    Private Sub diagram_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
        ' create the root of the tree
        rootNode = New ShapeNode(diagram)
        rootNode.Effect = New DropShadowEffect() With {.Opacity = 0.25}
        rootNode.AllowDrop = True
        rootNode.Bounds = New Rect(New Point(190, 60), NodeSize)
        rootNode.Text = "right-click me!"
        rootNode.Tag = 0
        rootNode.Selected = True
        diagram.Nodes.Add(rootNode)
    End Sub

    Private Shared ReadOnly NodeSize As New Size(70, 40)
End Class